# Generated by Django 2.2.1 on 2019-05-24 13:51

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0057_auto_20190524_1351'),
    ]

    operations = [
        migrations.AlterField(
            model_name='materials',
            name='location',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Locations'),
        ),
    ]
