# Generated by Django 2.2.2 on 2019-06-16 04:50

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0106_auto_20190614_1947'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_line',
            name='child',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='item_child', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='item_parent', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
    ]
