# Generated by Django 2.2.1 on 2019-05-29 10:22

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.code_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0028_auto_20190528_2223'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations',
            name='code',
            field=isc_common.fields.code_field.CodeField(),
        ),
        migrations.AddField(
            model_name='operations',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Operations'),
        ),
    ]
